package net.sendcloud.test;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.mail.Address;
import javax.mail.Header;
import javax.mail.MessagingException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMessage.RecipientType;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeUtility;

import org.apache.commons.io.FileUtils;

public class App {
	public static void main(String[] args) throws IOException, MessagingException {
		String eml = FileUtils.readFileToString(new File("d:/94736a3b-8cca-4f76-bdef-d60f52fb1a5b_MXBODY"), "UTF-8");
		// 附件保存路径，如果为空则保存到temp目录
		String path = "d:/";
		SendCloudMail scm = parse(eml, path);
		System.out.println(scm.getSubject());
		System.out.println(scm.getFrom());
		System.out.println(scm.getTo().get(0).getAddress());
		System.out.println(scm.getHtml());
		if (scm.getAttachments() != null) {
			for (File f : scm.getAttachments()) {
				System.out.println(f.getName());
			}
		}
	}

	/**
	 * 
	 * @param eml
	 * @param path 附件存储路径
	 * @return
	 * @throws MessagingException
	 * @throws IOException
	 */
	private static SendCloudMail parse(String eml, String path) throws MessagingException, IOException {
		SendCloudMail scm = new SendCloudMail();
		scm.setPath(path == null ? System.getProperty("java.io.tmpdir") : path);
		try (InputStream is = new ByteArrayInputStream(eml.getBytes())) {
			MimeMessage message = new MimeMessage(null, is);
			// subject
			String subject = message.getSubject();
			scm.setSubject(subject);
			// 邮件header
			Map<String, String> headerlist = new HashMap<String, String>();
			Enumeration<Header> allheaders = message.getAllHeaders();
			while (allheaders.hasMoreElements()) {
				Header h = allheaders.nextElement();
				headerlist.put(h.getName(), MimeUtility.decodeText(h.getValue()));
			}
			scm.setHeaders(headerlist);
			// 发信人
			String from = null, fromName = null;
			Address[] mailfrom = message.getFrom();
			if (mailfrom != null) {
				from = ((InternetAddress) message.getFrom()[0]).getAddress();
				fromName = ((InternetAddress) message.getFrom()[0]).getPersonal();
			}
			scm.setFrom(from);
			scm.setFromName(fromName);
			// 发信和收信时间
			scm.setSentDate(message.getSentDate());
			scm.setReceivedDate(message.getReceivedDate());
			// 收信人
			Address[] to = message.getRecipients(RecipientType.TO);
			if (to != null) {
				for (Address t : to)
					scm.addTo((InternetAddress) t);
			}
			// 抄送
			Address[] cc = message.getRecipients(RecipientType.CC);
			if (cc != null) {
				for (Address c : cc)
					scm.addCc((InternetAddress) c);
			}
			// 回复
			Address[] replyTo = message.getReplyTo();
			if (replyTo != null) {
				for (Address r : replyTo)
					scm.addReplyTo((InternetAddress) r);
			}
			parseContent(scm, message.getContent());
		}
		return scm;
	}

	/**
	 * html、text和附件解析
	 * 
	 * @param scm
	 * @param content
	 * @throws MessagingException
	 * @throws IOException
	 */
	private static void parseContent(SendCloudMail scm, Object content) throws MessagingException, IOException {
		if (content instanceof MimeMultipart) {
			MimeMultipart part = (MimeMultipart) content;
			int c = part.getCount();
			for (int i = 0; i < c; i++) {
				parseContent(scm, part.getBodyPart(i));
			}
		} else if (content instanceof MimeBodyPart) {
			MimeBodyPart body = (MimeBodyPart) content;
			if (body.getContentType().toLowerCase().indexOf("multipart/related") != -1
					|| body.getContentType().toLowerCase().indexOf("multipart/mixed") != -1
					|| body.getContentType().toLowerCase().indexOf("multipart/alternative") != -1) {
				if (body.getContent() instanceof MimeMultipart) {
					parseContent(scm, body.getContent());
				}
			} else if (body.getContentType().toLowerCase().indexOf("text/html") != -1
					&& body.getDisposition() == null) {
				scm.setHtml(body.getContent().toString());
			} else if (body.getContentType().toLowerCase().indexOf("text/plain") != -1
					&& body.getDisposition() == null) {
				scm.setText(body.getContent().toString());
			} else if (body.getContentType().toLowerCase().indexOf("application/") != -1) {
				String filename = MimeUtility.decodeText(body.getFileName());
				File f = new File(scm.getPath() + filename);
				FileUtils.copyInputStreamToFile((InputStream) body.getContent(), f);
				scm.addAttachment(f);
			}
		} else if (content instanceof String) {
			scm.setHtml(content.toString());
		}
	}
}